import logging
import typing
from pathlib import Path

from .page import Page
from .feeds import RSSFeed


class Collection:
    engine = ""
    """Collection objects serve as a way to quickly process pages that have a
    LARGE portion of content that is similar or file driven.

    The most common form of collection would be a Blog, but can also be
    static pages that have their content stored in a dedicated file.

    Currently, collections must come from a content_path and all be the same
    content type.


    Example
    -------
    from render_engine import Collection

    @site.register_collection()
    class BasicCollection(Collection):
        pass


    Attributes
    ----------
    engine: str, optional
        The engine that the collection will pass to each page. Site's default
        engine
    template: str
        The template that each page will use to render
    routes: List[str]
        all routes that the file should be created at. default []
    content_path: List[PathString], optional
        the filepath to load content from.
    includes: List[str], optional
        the types of files in the content path that will be processed
        default ["*.md", "*.html"]
    has_archive: Bool
        if `True`, create an archive page with all of the processed pages saved
        as `pages`. default `False`
    _archive_template: str, optional
        template filename that will be used if `has_archive==True` default: archive.html"
    _archive_slug: str, optional
        slug for rendered page if `has_archive == True` default: all_posts
    _archive_content_type: Type[Page], optional
        content_type for the rendered archive page
    _archive_reverse: Bool, optional
        should the sorted `pages` be listed in reverse order. default: False

    """
    engine = ''
    page_content_type = Page
    content_path = "content"
    template = "page.html"
    includes = ["*.md", "*.html"]
    routes = [""]
    has_archive = False
    _archive_template = "archive.html"
    _archive_slug = "all_posts"
    _archive_content_type = Page
    _archive_reverse = False

    @staticmethod
    def _archive_default_sort(cls):
        """attribute pulled from a rendered Page to sort `pages`"""
        return cls.slug

    @property
    def pages(self) -> typing.List[typing.Type[Page]]:
        """Iterate through set of pages and generate a `Page`-like object for each."""
        pages = []

        for i in self.includes:
            for _file in Path(self.content_path).glob(i):
                page = self.page_content_type(content_path=_file)
                page.routes = self.routes
                page.template = self.template
                pages.append(page)

        return pages

    @property
    def archive(self):
        """Create a `Page` object for those items"""
        archive_page = self._archive_content_type()
        archive_page.template = self._archive_template
        archive_page.slug = self._archive_slug
        archive_page.engine = ""
        archive_page.title = self.__class__.__name__
        archive_page.pages = sorted(
            self.pages,
            key=lambda p: self._archive_default_sort(p),
            reverse=self._archive_reverse,
        )
        return archive_page



from typing import List, Any
from dataclasses import dataclass

@dataclass
class Link:
    """
    An opinionated format to reference links. Great for creating headers

    Attributes
    ----------
    name : str
        the content of the link. 

    url : str
        the href that points to a webpage

    links : List[Any]
        Can be used to embed other link objects (userful for submenus)

    image : str, optional
        the source url for the image link

    icon : str, optional
        similar to image, but allows for custom html for icons
    """
    name: str
    url: str
    links: list = List[Any]
    image: str = ''
    icon: str = ''



import logging

import re
import typing
from typing import List
from pathlib import Path

from jinja2 import Markup
from markdown import markdown

from ._type_hint_helpers import PathString

# default matching param for posts
base_matcher = re.compile(r"(^\w+: \b.+$)", flags=re.M)


def parse_content(content: str, matcher=base_matcher):
    """
    split content into attributes and content text

    Parameters:
        content : str
            The content to be parsed
        matcher : str, optional
            A compiled regular expression that splits the content.
            default `base_matcher`
    """

    parsed_content = re.split(matcher, content)
    content = parsed_content.pop().strip()
    attrs = list(filter(lambda x: x.strip(), parsed_content))
    return attrs, content


class Page:
    """
    Base component used to make web pages.

    All components that represent content in HTML, XML, or JSON generated by 
    Render Engine should be a Page object.

    Pages can be rendered directly from a template or from a file.

    Page objects can be used to extend existing page objects.


    Examples:
        ```
        # Basic Page with No Template Variables
        @site.register_route('basic_page.html')
        class BasicPage(Page):
            template = 'template_file.html' # user provided template

        # Basic Page with Variables
        @site.register_route('page_with_vars')
        class PageWithVars(Page):
            title = 'Site Title'

        # Page Loading from File
        @site.register_route('page_from_file')
        class PageFromFile(Page):
            content_path = 'index.md' # loaded from content path can be '.md' or '.html'

        # Page Inherited from Other Page
        @site.register_route('basic_page.html')
        class BasicPage(Page):
            template = 'template_file.html' # user provided template
            title = 'Base Page'

        @site.register_route('other_page.html')
        class InheritingPage(Page):
            # template will be inherited from the BasicPage
            title = 'Inherited Page'
        ```

    Attributes:
        engine: str, optional
            The engine that the Site should refer to or the site's default engine
        template: str
            The template that the Site should refer to. If empty, use site's default
        routes: List[str]
            all routes that the file should be created at. default []
        content_path: List[PathString], optional
            the filepath to load content from.
        slug: str
            The engine's default route filename
        content: str
            preprocessed text stripped from initialized content. This will not
            include any defined attrs
        html: str
            text converted to html from _content
    """

    engine = ""
    template = ""
    routes = [""]

    def __init__(
        self, content_path: PathString = None, content: str = "", matcher=base_matcher
    ):
        """
        If a content_path exists, check the associated file, processing the
        vars at the top and restitching the remaining lines

        Parameters:
            content_path: List[PathString], optional
                the filepath to load content from.
            content: str, optional
                raw text to be processed into HTML
            matcher: str, optional
                A compiled regular expression that splits the content.
                defatul `base_matcher`

        TODOs:
            - ADD Documentation for attrs/content
            - Make Slug Conversion Smarter

        """

        self.content_path = content_path

        if content_path:
            content = Path(content_path).read_text()

        valid_attrs, self._content = parse_content(content, matcher=matcher)

        for attr in valid_attrs:
            name, value = attr.split(": ", maxsplit=1)
            setattr(self, name.lower(), value.strip())

        if not hasattr(self, "slug"):
            self.slug = getattr(self, "title", self.__class__.__name__)

        self.slug = self.slug.lower().replace(" ", "_")

    def __str__(self):
        return self.slug

    @property
    def html(self):
        """Text from self._content converted to html"""

        if hasattr(self, "_content"):
            return markdown(self._content)

        else:
            return ""

    @property
    def content(self):
        """html = rendered html (not marked up). Is `None` if `content == None`"""
        if self._content:
            return Markup(self.html)

        else:
            return ""



import logging
import typing
import pendulum

from .blog import BlogPost, Blog
from .site import Site
from .feeds import RSSFeedItem, RSSFeedEngine


class MicroBlogPost(BlogPost):
    """
    Page Like Object with slight modifications to work with BlogPosts

    Attribtues:
        title : str
            default ''. Leave blank.

        slug : str
            the name for the file for that will

        rss_feed_item : RSSFeedItem
            the content in an rss format
    """

    title = ""

    def __init__(self, **kwargs):
        """checks published options and accepts the first that is listed"""
        super().__init__(**kwargs)
        self.slug = pendulum.parse(self.date_published, strict=False).format("YMDHmS")

    @property
    def rss_feed_item(self):
        feed_item = RSSFeedItem(self)
        return feed_item


class MicroBlog(Blog):
    """
    Custom Blog Class pointing to custom templates
    """
    _archive_template = "microblog_archive.html"
    _archive_slug = "all_microblog_posts"
    page_content_type: typing.Type[MicroBlogPost] = MicroBlogPost
    _archive_reverse: bool = True
    has_archive = True



import render_engine
import click
from jinja2 import Environment, PackageLoader

import shutil
from pathlib import Path


def create_templates_directory(
    base_dir: str = Path().cwd(),
    templates_dir: str = "templates",
    include_template_files: bool = True,
):
    """
    Create Base Folders for your
    output_dir, static_dir, content_path, and templates_path
    """

    source_folder = Path(Path(render_engine.__file__).parent, "templates")
    templates_path = Path(base_dir).joinpath(templates_dir)

    if include_template_files:
        return shutil.copytree(source_folder, templates_path, dirs_exist_ok=True)

    else:
        return Path(base_dir).joinpath(content_path).mkdir(parents=True, exist_ok=True)


@click.command()
@click.option(
    "--base-directory", default=".", help="starting directory for your project"
)
@click.option(
    "--static-directory", default="static", help="static directory for your project"
)
@click.option(
    "--content-directory", default="content", help="content directory for your project"
)
@click.option(
    "--static-path/--no-static-path", "-s/-S", default=True, help="create a static path"
)
@click.option(
    "--content-path/--no-content-path",
    "-c/-C",
    default=True,
    help="directory for prerendered files",
)
@click.option(
    "--templates-path/--no-templates-path",
    "-t/-T",
    default=True,
    help="directory for template files",
)
@click.option(
    "--templates-files/--no-templates-files",
    "-f/-F",
    default=True,
    help="files for templates-directory",
)
@click.option(
    "--blog",
    is_flag=True,
    prompt="Would you like to create a blog object",
    help="Add a Blog Object",
)
@click.option("--microblog", is_flag=True, default=False)
@click.option(
    "--pages-directory",
    is_flag=True,
    help="create a separate pages directory inside of your content path",
)
@click.option("--templates-directory", default='templates',
        help="directory for template files")
def _main(
    base_directory,
    static_path,
    content_path,
    templates_path,
    blog,
    microblog,
    templates_files,
    static_directory,
    content_directory,
    pages_directory,
    templates_directory,
):
    """
    CLI that allows folks to quickly build their starting directory
    """

    if static_path:
        Path(base_directory).joinpath(static_directory).mkdir(
            parents=True, exist_ok=True
        )

    if content_path:
        Path(base_directory).joinpath(content_directory).mkdir(
            parents=True, exist_ok=True
        )

    if templates_path:
        create_templates_directory(
            base_directory, templates_directory, templates_files
        )


if __name__ == "__main__":
    _main()



import logging
import os
import shutil
import typing
from pathlib import Path

from ._type_hint_helpers import PathString
from .collection import Collection
from .engine import Engine
from .feeds import RSSFeedEngine
from .page import Page


class Site:
    """
    The site stores your pages and collections to be rendered.

    Pages are stored and created with `site.render()`.
    Collections are stored to be used for future use.
    Sites also contain global variables that can be applied in templates.

    Attributes:
        routes (list):
            storage of registered_routes
        collections (dict):
            storage of registered collections
        output_path (str or pathlib.Path):
            the path to directory which all rendered html pages will be stored.
            default `./output`
        static_path (str or pathlib.Path):
            the path to directory for static content. This will be copied over
            into the `output_path`
        SITE_TITLE (str):
            configuration variable title of the site. This is only used in your
            environment template variables. While Optional you will be warned
            if you do not supply a new variable. default: 'Untitled Site'
        SITE_URL (str):
            configuration variable url of the of the site. While Optional you will be
            warned if you do not supply a new variable. default: 'Untitled Site'
            default 'https://example.com'

    Todo:
        - remove SITE_LINK
        - make SITE_URL accesible as a Page variable and allow for switch for
            Relative and Absolute URLS
    """

    routes: typing.List[str] = []
    collections = {}
    output_path: Path = Path("output")
    static_path: Path = Path("static")
    SITE_TITLE: str = "Untitled Site"
    SITE_LINK: str = "https://example.com"
    SITE_URL: str = "https://example.com"

    def __init__(self, strict: bool = False):
        """
        Clean Directory and Prepare Output Directory

        Parameters:
            routes (list):
                storage of registered_routes
            collections (dict):
                storage of registered collections
            output_path (str or pathlib.Path):
                the path to directory which all rendered html pages will be stored.
                default `./output`
            static_path (str or pathlib.Path):
                the path to directory for static content. This will be copied over
                into the `output_path`
            strict (bool):
        """

        # Make Output Path if it doesn't Exist
        self.output_path = Path(self.output_path)

        # strict deletes the directory and rebuilds it from scratch
        if strict and self.output_path.is_dir():
            shutil.rmtree(self.output_path)

        # copy a defined static path into output path
        if Path(self.static_path).is_dir():
            self.output_path.mkdir(exist_ok=True)
            shutil.copytree(
                self.static_path,
                self.output_path.joinpath(self.static_path),
                dirs_exist_ok=True,
            )

        self.engines: typing.Dict[str, typing.Type[Engine]] = {
                "default_engine": Engine(),
                "rss_engine": RSSFeedEngine(),
        }

        # check for errors
        if self.SITE_TITLE == "Untitle Site":
            logging.warning(f"No custom site title defined. Using the {self.SITE_TITLE=}")

        if self.SITE_URL == self.SITE_LINK == "https://example.com":
            logging.warning(f"No custom site URL defined. Using the {self.SITE_URL=}")

    def register_collection(self, collection_cls: typing.Type[Collection]) -> None:
        """
        Add a class to your `self.collections`
        iterate through a classes `content_path` and create a classes
        `Page`-like objects, adding each one to `routes`.

        Use a decorator for your defined classes.

        Examples:
            ```
            @register_collection
            class Foo(Collection):
                pass
            ```

        Args:
            collection_cls (Collection):
                Collection to parse
        """
        collection = collection_cls()
        self.collections.update({collection.__class__.__name__: collection})

        for page in collection.pages:
            self.route(cls=page)

        if collection.has_archive:
            self.route(cls=collection.archive)

            if hasattr(collection, 'feeds'):
                for feed in collection.feeds:
                    if feed:
                        self.register_feed(feed=feed, collection=collection)

    def register_feed(self, feed, collection: Collection) -> None:
        _feed = feed()
        _feed.slug = ''.join([collection.__class__.__name__.lower(), _feed.slug])
        _feed.items = [page.rss_feed_item for page in collection.pages]
        _feed.title = ' - '.join([self.SITE_TITLE, _feed.title])
        _feed.link = ''.join([self.SITE_LINK, _feed.link])

        self.route(cls=_feed)


    def route(self, cls) -> None:
        self.routes.append(cls)

    def register_route(self, cls) -> None:
        self.routes.append(cls())


    def render(self, dry_run: bool = False) -> None:
        for page in self.routes:
            engine = self.engines.get(page.engine, self.engines["default_engine"])
            content = engine.render(page, **vars(self))


            for route in page.routes:
                logging.info(f'starting on {route=}')
                route = self.output_path.joinpath(route.strip("/"))
                route.mkdir(exist_ok=True)
                filename = Path(page.slug).with_suffix(engine.extension)
                filepath = route.joinpath(filename)

                if not dry_run:
                    filepath.write_text(content)

                else:
                    print(f'{content} writes to {filepath}')




import logging
from pathlib import Path
from typing import Optional, Sequence, Type

import jinja2
from jinja2 import FileSystemLoader, select_autoescape

from .page import Page


class Engine:
    """
    This is the engine that is builds your static site.
    Use `Engine.run()` to output the files to the designated output path.

    Attributes:
        extension : str
            the extension to use in the rendered files
            default '.html'
        environment : Any
            the environment renderer that you want to use. You can use any environment that you like. Environments
            should support a `get_template` and `render`

    Todos:
        * Create default template
        * Method to build template directory
    """

    extension: str = ".html"
    environment = jinja2.Environment(
        autoescape=select_autoescape(), loader=FileSystemLoader("templates")
    )

    def get_template(self, template: str):
        """
        fetches the requested template from the environment. Purely a
        convenience method

        Parameters:
            template : str
                the template file to look for
        """
        return self.environment.get_template(template)

    def render(self, page: Type[Page], **kwargs):
        """
        generates the rendered HTML from from environment

        Parameters:
            page : Page
                the page object to render into html
        """
        if page.template:
            template = self.get_template(page.template)
            kwargs.update({"content": page.content})
            kwargs.update(page.__dict__)

            return template.render(**kwargs)

        else:
            return page.content



import logging
import typing
from typing import List

import pendulum
from more_itertools import first_true

from .collection import Collection
from .page import Page
from .feeds import RSSFeedItem, RSSFeed, RSSFeedEngine
from .site import Site


class BlogPost(Page):
    """
    Page Like object with slight modifications to work with BlogPosts.

    Attributes:
        template : str
            the default template that the site will look for
        rss_feed_item : RSSFeedItem
            the content in an rss format
        date : pendulum.datetime
            date parsed in datetime format. usesul for sorting and things
        date_published : str
            date formated for `RSSFeed`
        date_friendly : str
            an easy to read string version of the date

    """

    template = "blog_post.html"

    def __init__(self, **kwargs):
        """
        checks published options and accepts the first that is listed

        Attributes:
            date : pendulum.datetime
                date parsed in datetime format. usesul for sorting and things
            date_published : str
                date formated for `RSSFeed`
            date_friendly : str
                an easy to read string version of the date
        """

        super().__init__(**kwargs)
        # Add some flexibility to date detection
        date = first_true([
                getattr(self, 'date_modified', None),
                getattr(self, 'modified_date', None),
                getattr(self, 'date_published', None),
                getattr(self, 'publish_date', None),
                getattr(self, 'date'),
                ])
        parsed_date = pendulum.parse(date, strict=False)
        self.date = parsed_date.set(tz=pendulum.local_timezone())
        self.date_published = self.date.to_rfc2822_string()
        self.date_friendly = self.date.format('MMM DD, YYYY HH:mm A')

    @property
    def rss_feed_item(self):
        return RSSFeedItem(self)


class Blog(Collection):
    """
    Custom Collection Class with Archiving Enabled and the RSS Feed

    Todos:
        - Add Support for JSON Feeds
        - Rename the archive items so they are not private
    """

    page_content_type: typing.Type[BlogPost] = BlogPost
    _archive_reverse: bool = True
    has_archive: bool = True
    feeds = [RSSFeed]

    @staticmethod
    def _archive_default_sort(cls):
        """
        How to sort pages

        Attributes:
            _archive_default_sort : Any
                default BlogPost.date

        Todos:
            - Rename the archive items so they are not private
        """
        return cls.date



"""
The Feeds Logic That Makes Up RSS and ATOM FeedTypes.

This is the base files and should only contain the params identified by the
standards defined.

RSS: http://www.rssboard.org/rss-specification
JSON: https://jsonfeed.org/version/1
"""

import logging

import jinja2
import pendulum
from jinja2 import PackageLoader, select_autoescape
from more_itertools import first_true

from .engine import Engine
from .page import Page


class RSSFeedItem:
    """
    The Object to be used with an RSS Feed.

    Attributes:
        title (str):
            Title of the post
        description (str):
            Content of the post. Can be the post's (in order) `description`,
            `content` or `summary`.
        guid (str):
            unique identifier of the content
        link (str): link to the post. Due to the design of the system, use the
            reference link and expand to the full link using information from the `site`
        pub_date (str):
            datetime formatted to RFC 822 (or 2822)

    .. _should conform to the RSS 2.0 specification:
       <https://cyber.harvard.edu/rss/rss.html>
    """

    def __init__(self, cls):
        '''
        Parse information from the given class object.

        Raises:
            AttributeError:
        '''

        self.title = getattr(cls, "title", "")
        # Posts will have one or many of the following
        description = getattr(cls, "description", None)
        content = getattr(cls, "content", None)
        summary = getattr(cls, "summary", None)
        # Set description to the preference
        self.description = first_true([description, content, summary], None)

        # Raise Error if you have neither a title or description
        if not self.title and not self.description:
            error_msg = "Your page must have either a title or a description"
            raise AttributeError(error_msg)

        self.guid = getattr(cls, "guid", cls.slug)
        self.link = cls.slug
        self.pub_date = cls.date_published


class RSSFeed(Page):
    """The RSS Feed Component of an Archive Object"""

    template = "rss2.0.rss"
    engine = "rss_engine"
    title = "RSS Feed"
    link = ""
    slug = ""


class RSSFeedEngine(Engine):
    """The Engine that Processes RSS Feed"""

    extension = ".rss"
    environment = jinja2.Environment(
        loader=PackageLoader("render_engine", "rss"),
        autoescape=select_autoescape(),
        trim_blocks=True,
    )
